 # Name :          	CompLayerScene ###
# Date :	      	  08 31 2012
# Description :   	Comp2Layer/Scene: (Select from context menu unless otherwise specified)
#                   Place the Component on a new layer named from the component
#                 	Layer name has optional prefix to organize assembly layers in a group
#				  	        All instances of the component will be placed on the new layer
#                 	User will have the option for creating a Drawing Scene
#                     A second new Drawing Layer named "Z-##" (01 thru 99) will be created.
#                 	  A copy of the Board will be added to the Z-## Layer for the Drawing Scene
#				  	          Then the front, top, bottom, Lside & Rside views are created.
#                     The program creates these veiws and no longer requires 'makeOrthoViews'
#                 	  The drawing Scene will be updated
#                   
#                   NestedComp2Scene: to make layer and scene for Nested Component
#                   SubAssy2Scene: to make Sub Assembly scene for grouped components
#                   Comp2Scene: to make Drawing Scene for Component on a new Drawing Layer
#                   DwgViews: to make drawing views of component on the same visible layer 
#                   Set Comp2Scene Pref: to allow user to set Preferences [Plugins/Set Comp2Layer Pref]
#                   New Hidden Layer: to create a layer hidden in all scenes. [Plugins/New Hidden Layer]
#
# Author :       	  Keith Krueger #kbkrueger2@gmail.com# (with a big help from plugin forum:)
# 

require('sketchup.rb')


# Load Misc K2WS files
require(File.join(File.dirname(__FILE__), 'k2ws_module.rb'))#-NameSpace Module

# Test to see if Old Version with different file & dir names exist and delete if present!
if (FileTest.exist?(File.join(File.dirname(File.dirname(__FILE__)), 'K2WS_Comp2LayerScene_ext.rb')))
  require(File.join(File.dirname(__FILE__), 'deleteK2WS.rb'))
  UI.messagebox("IMPORTANT:  AN UPDATE was required & Old Files Deleted \n Restart SketchUp to Complete Installation\n  Comp2LayerScene Plugin Ver 2.0.0 ")
end
if (FileTest.exist?(File.join(File.dirname(File.dirname(__FILE__)), 'K2WS_Comp2LayerScene_ext.rb!')))
  require(File.join(File.dirname(__FILE__), 'deleteK2WS.rb'))
  UI.messagebox("IMPORTANT:  AN UPDATE was required & Old Files Deleted \n Restart SketchUp to Complete Installation\n  Comp2LayerScene Plugin Ver 2.0.0 ")
end


module K²WS::Comp2LayerScene

  # Set Constantants
  C2LAYSCENE = "Comp2Layer/Scene"
  NC2SCENE = "NestedComp2Scene"
  GP2SCENE = "SubAssy2Scene"
  C2SCNE = "Comp2Scene"
  DWGVIEWS = "DwgViews"
  SETPREF = "Set Comp2Scene Pref"
  HIDLAYER = "New Hidden Layer"
  
  
  class << self
    
    ############ Use this to place NEW COMPONENT on its OWN LAYER with Option to CREATE SCENE ##################
    
    def comp2_layer_scene_command 
      # Model Information
      self.get_model_info
      entity = @ss.first
      ent_def = entity.definition
      self.read_data
      self.main_view_check
      
      # This section gets a Component Name from User
      if @ans[2] == "Yes"
        cName = @ss[0].definition.name
        ps = "Enter Component Name "
        dv = cName
        ts = "Get Name for Component"
        @user_name = get_user_name(ps,dv,ts)
        if @user_name != cName then @ss[0].definition.name = @user_name end
        # Test name to determine if unique
        while @ss[0].definition.name != @user_name
          UI.messagebox("Component \"#{@user_name}\" exists,make unique.")
          dv = @user_name
          @user_name = self.get_user_name(ps,dv,ts)
          @ss[0].definition.name = @user_name
        end # while @ss[0]
      else
        @user_name = @ss[0].definition.name
      end # if @ans[2]
      
      # Create new layer for Component in Assembled Position 
      if @ans[0] == "None" || @ans[0] == "none"
        @layer_name = @user_name
      else
        @layer_name = @ans[0].to_s + "-" + @user_name
      end # if @ans[0] == "Yes"
      #Debug UI.messagebox"Layer Name  = " + @layer_name
      
      # Test Component Layer Name to determine if unique
      @layer_name = self.test_layerName(@layer_name)
      
      
  Sketchup.active_model.start_operation "Comp2_Layer_Scene"
      
      self.create_layer(@layer_name)  # Creates Layer for Component
      
      #Add Component instances to new layer	
      @model.selection.add(entity.definition.instances)
      nosel=@ss.length
      #layers = model.layers
      i = 0          
      loop do
        #Debug  UI.messagebox("length = " + nosel.to_s)
        entity=@ss[i-1]
        #Debug  UI.messagebox("Enity Layer Name = " + entity.layer.name)
        if entity.layer.name[0,1] == "Z" && entity.layer.name[3,1] == "-"
          #Do not move enity from Drawing Layer 
        else
          entity.layer=(@layers[@layer_name])
        end
        i=i + 1
        break if i==nosel          
      end # loop
             
      # Make New Part Component visible in Main Assembly Scene
      @pages.selected_page = @model.pages[@mv_name]
      page = @pages.selected_page
      page.set_visibility(@layers[@layer_name],true)
      status = page.update
      #Debug UI.messagebox("Main View status = " + status.to_s)
      
      # Note: Scene & Page both refer to the same thing!!!
      ## Ask user if Drawing Scene should be added
      result=UI.messagebox "Add Scene for Comp Drawing? ", MB_YESNO 			
      if result==6  #Add Scene for Drawing and place copy of component in Scene
        # Create new layer for Part Drawing Scene
        layerno = self.get_Drawing_layer_No
        @dwg_layer_name = "Z" + layerno + "-" + @user_name 
        
        # Test Drawing Layer name to determine if unique
        @dwg_layer_name = self.test_layerName(@dwg_layer_name)
        self.create_layer(@dwg_layer_name)  # Creates Layer for Drawing Scene
        
        #Insert copy @ orign [Code provided by TIG from SU Forum]
        insert_tran = Geom::Transformation.new [@offset,@offset,0]
        gp=@model.active_entities.add_group(entity)
        cp=@model.active_entities.add_instance(gp.entities.parent, insert_tran)
        it = cp.transformation
        gp.explode
        c2=cp.entities[0]
        cp.explode
      
        #Setup Part on Drawing View for Landscape Printing:  Red axis Length, Blue axis Width
        
        #Transform and rotate component so largest dim [Length] is on the red axis
        selection_bb = entity.bounds
        self.get_bounds_max_dir(selection_bb)
        self.get_trans
        if @max_dir != "red axis" then c2.transform! @f_tran  end
        
        #Transform and rotate component so second largest dim [Width] is on the blue axis
        selection_bb = entity.bounds
        if selection_bb.height > selection_bb.depth
          width_tran = Geom::Transformation.rotation [0,0,0], [1,0,0], 90.degrees
          c2.transform! width_tran
        end #if height
        c2_t = c2.transformation
        selection_bb = entity.bounds
        
        # Place copy on Drawing layer
        c2.layer=(@layers[@dwg_layer_name])
        
        #Get Scene Name for drawing scene
        if @ans[1] == "Yes" then pn = "D - " + @dwg_layer_name[1,2] end
        if @ans[1] == "No" then pn = @user_name end
      
        # Test Scene name to determine if unique
        loop = "yes"
        while loop == "yes"
          loop = "no"
          @pages.each {|page|
            if page.name == pn
            UI.messagebox("Page \"#{pn}\" exists,make unique.")
            ps = "Make Page Name Unique!!"
            dv = pn
            ts = "Page Name for Component"
            pn = self.get_user_name(ps,dv,ts)
            loop = "yes"
            end # if page
          }
        end # while
        
        # Add Page & Description for Drawing Scene
        self.add_page(pn,@user_name,@dwg_layer_name)
        page = @pages[@user_name]
        
        #Add Component Name as Text to Drawing Scene
        name_text = @ents.add_text @user_name, [selection_bb.center.x,selection_bb.center.y,(selection_bb.center.z - (selection_bb.depth * 0.7))]
        name_text.layer=(@layers[@dwg_layer_name]) 
       
        #This section adds Drawing Views & Set up Drawing Scene
        @ss.clear #Clear selection so only one component will be selected
        @ss.add c2  #Add instance in Scene to selection
        self.drawing_views(ent_def,@dwg_layer_name,c2_t,c2)
        Sketchup.active_model.active_view.camera.perspective = false
        Sketchup.send_action "viewFront:"
        @view.zoom @ss
        @view.refresh
        Sketchup.send_action "pageUpdate:"

      end #if results==Yes
      
  Sketchup.active_model.commit_operation # Comp2_Layer_Scene
   
      @ss.clear
  
    end # def comp2_layer_scene_command
    #----------------------------------------------------------------------------------#
    
    
    ############ Use to create SECENE AND LAYER FOR NESTED COMPONENT ######################
    
    def nested_component_command(obj)
      self.get_model_info
      self.read_data
      self.main_view_check
      entity = obj
      @obj_parent_name = obj.parent.name
      @layer_name = obj.parent.layer.name
      ent_def = entity.definition
      Sketchup.active_model.close_active() # Closes component opened to select Nested Component!!

      #Drawing Layer Name for Nested Component
      @nc_name=obj.definition.name
      ps = "Enter Drawing Layer Name "
      dv = @nc_name + " " + @obj_parent_name
      ts = "Get Drawing Layer Name for Nested Component"
      @user_name = get_user_name(ps,dv,ts)
      
      #Set Prefix for Drawing Layer Name for Nested Component 
      layerno = self.get_Drawing_layer_No
      @dwg_layer_name = "Z" + layerno + "-" + @user_name
    
      # Test name to determine if unique
      while @layers[@dwg_layer_name]
        UI.messagebox("Layer \"#{@dwg_layer_name}\" exists,make unique.")
        ps = "Make Layer Name Unique!!"
        dv = @dwg_layer_name
        ts = "Nested Component Layer Name"
        @dwg_layer_name = self.get_user_name(ps,dv,ts)
      end
      
  Sketchup.active_model.start_operation "Nested_Comp_Scene" 
      
      # Create new layer for Part Drawing Scene
      self.create_layer(@dwg_layer_name)  # Creates Layer for Drawing Scene
      
      #Origional Code
      #Code provided by TIG from SU Forum  Note: This fails when used for nested copy
      insert_tran = Geom::Transformation.new [@offset,@offset,0]
      c2=@model.active_entities.add_instance(ent_def,insert_tran)

      #Transform and rotate component so largest dim [Length] is on the red axis
      selection_bb = entity.bounds
      self.get_bounds_max_dir(selection_bb)
      self.get_trans
      if @max_dir != "red axis" then c2.transform! @f_tran  end
      
      #Transform and rotate component so second largest dim [Width] is on the blue axis
      selection_bb = entity.bounds
      if selection_bb.height > selection_bb.depth
        width_tran = Geom::Transformation.rotation [0,0,0], [1,0,0], 90.degrees
        c2.transform! width_tran
      end #if height
      c2_t = c2.transformation
      selection_bb = entity.bounds
      
      # Place copy on Drawing layer
      c2.layer=(@layers[@dwg_layer_name])
      
      #Get Scene Name for drawing scene
      if @ans[1] == "Yes" then pn = "D - " + @dwg_layer_name[1,2] end
      if @ans[1] == "No" then pn = @user_name end
    
      # Test Scene name to determine if unique
      loop = "yes"
      while loop == "yes"
        loop = "no"
        @pages.each {|page|
          if page.name == pn
          UI.messagebox("Page \"#{pn}\" exists,make unique.")
          ps = "Make Page Name Unique!!"
          dv = pn
          ts = "Page Name for Component"
          pn = self.get_user_name(ps,dv,ts)
          loop = "yes"
          end # if page
        }
      end # while
      
      # Add Page & Description for Drawing Scene
      self.add_page(pn,@user_name,@dwg_layer_name)
      page = @pages[@user_name]
      
      #Add Component Name as Text to Drawing Scene
      ## Not Available for Nested Components
      
      #This section adds Drawing Views & Set up Drawing Scene
      @ss.clear #Clear selection so only one component will be selected
      @ss.add c2  #Add instance in Scene to selection
      self.drawing_views(ent_def,@dwg_layer_name,c2_t,c2)
      Sketchup.active_model.active_view.camera.perspective = false
      Sketchup.send_action "viewFront:"
      @view.zoom @ss
      @view.refresh
      Sketchup.send_action "pageUpdate:"
      
  Sketchup.active_model.commit_operation # Nested_Comp_Scene
  
      @ss.clear
        
    end # def nested_component_command 
    #----------------------------------------------------------------------------------#
    
    
    ############ Use to create SCENE and Layer for SUB ASSEMBLY GROUP #####################
    
    def sub_assembly_command
      self.get_model_info
      self.read_data
      self.main_view_check
      ssa=@ss.to_a
      @ss.clear
      @group=@ents.add_group(ssa)
      @groupcopy = @group.copy 
      ent_def=@groupcopy.entities.parent
      @groupcopy.name = "SA-"
      @group.explode
      
      # Set variables to get layer name
      ps = "Enter Name for SubAssy Layer"
      dv = @groupcopy.name
      ts = "Get Name for Sub-Assembly Layer"
      @layer_name = self.get_user_name(ps,dv,ts)
      
      # Test name to determine if unique
      while @layers[@layer_name]
        UI.messagebox("Layer \"#{@layer_name}\" exists,make unique.")
        @layer_name = self.get_user_name(ps,dv,ts)
      end
      
  Sketchup.active_model.start_operation "SubAssy_Scene"
      
      self.create_layer(@layer_name)
       
      # Add group and individual components to New Layer
      @groupcopy.layer = @layer_name
      @groupcopy.entities.to_a.each{|e|e.layer = @layer_name}
      @groupcopy.name = @layer_name.to_s
      
      # Find current entity bounds with the largest dim
      selection_bb = @groupcopy.bounds
      self.get_bounds_max_dir(selection_bb)
      
      # Transform and rotate component so largest dim [Length] is on the red axis
      if @max_dir != "red axis" then @groupcopy.transformation = self.get_trans end
      
      # Transform and rotate component so second largest dim [Width] is on the blue axis
      selection_bb = @groupcopy.bounds
      if selection_bb.height > selection_bb.depth
        width_tran = Geom::Transformation.rotation [0,0,0], [1,0,0], 90.degrees
        @groupcopy.transform! width_tran
      end #if height
      @groupcopy_t = @groupcopy.transformation
      
      #Offset group copy
      offset_transformation = Geom::Transformation.new [@offset,@offset,0]
      @groupcopy.transformation = offset_transformation * @groupcopy_t
      @groupcopy_t = @groupcopy.transformation
      
      # Add page for drawing & Make SubAssy Group visible on Drawing Page
      pd = @layer_name; pn = @layer_name; ln = @layer_name
      self.add_page(pn,pd,ln)
      
      #Add Title as Text to Scene
      selection_bb = @groupcopy.bounds
      scene_title = @layer_name
      name_text = @entities.add_text scene_title, [selection_bb.center.x, selection_bb.center.y, (selection_bb.center.z - (selection_bb.depth * 0.7))]
      name_text.layer=(@layers[@layer_name])
      
      #This section adds Drawing Views & Set up Drawing Scene
      @ss.clear #Clear selection so only one component will be selected
      @ss.add @groupcopy  #Add instance in Scene to selection
      self.drawing_views(ent_def,@layer_name,@groupcopy_t,@groupcopy)
      Sketchup.active_model.active_view.camera.perspective = false
      Sketchup.send_action "viewFront:"
      @view.zoom @ss
      @view.refresh
      Sketchup.send_action "pageUpdate:"
      
  Sketchup.active_model.commit_operation # SubAssy_Scene
  
      @ss.clear
        
    end # def sub_assembly_command
    #----------------------------------------------------------------------------------#
    
    
    
    ############ Use to create SCENE ONLY for component ALREADY ON component layer  AND/OR  ##############
    ############ Use to create SCENE on NEW SCENE LAYER for MIRROR COMPONENT (component was flipped) #####
    
    def comp2_Scene_command
    
      self.get_model_info
      self.read_data
      self.main_view_check
      entity = @ss.first
      ent_def = entity.definition
      cName=@ss[0].definition.name
      parent = entity.parent
      cur_layer_name = @ss[0].layer.name
      self.read_data
      @user_name = @ss[0].definition.name
      @layer_name = entity.layer.name
      
          
  Sketchup.active_model.start_operation "Comp2_Scene"
      
      # Create new layer Name for Part Drawing Scene and Make New Layer
      
      layerno = self.get_Drawing_layer_No
      @dwg_layer_name = "Z" + layerno + "-" + @user_name
      
      # Test Layer name to determine if unique
      pn = @dwg_layer_name
      loop = "yes"
      while loop == "yes"
        loop = "no"
        @layers.each {|layer|
          if layer.name == pn
          UI.messagebox("Layer \"#{pn}\" exists,make unique.")
          ps = "Make Layer Name Unique!!"
          dv = pn
          ts = "Layer Name for Drawing Scene"
          pn = self.get_user_name(ps,dv,ts)
          loop = "yes"
          end # if layer.name
        }
      @dwg_layer_name = pn  
      end # while
    
      self.create_layer(@dwg_layer_name)  # Creates Layer for Drawing Scene
      
      #Insert copy @ orign [Code provided by TIG from SU Forum]
      insert_tran = Geom::Transformation.new [@offset,@offset,0]
      gp=@model.active_entities.add_group(entity)
      cp=@model.active_entities.add_instance(gp.entities.parent, insert_tran)
      it = cp.transformation
      gp.explode
      c2=cp.entities[0]
      cp.explode
    
      #Transform and rotate component so largest dim [Length] is on the red axis
      selection_bb = entity.bounds
      self.get_bounds_max_dir(selection_bb)
      self.get_trans
      if @max_dir != "red axis" then c2.transform! @f_tran  end
      
      #Transform and rotate component so second largest dim [Width] is on the blue axis
      selection_bb = entity.bounds
      if selection_bb.height > selection_bb.depth
        width_tran = Geom::Transformation.rotation [0,0,0], [1,0,0], 90.degrees
        c2.transform! width_tran
      end #if height
      c2_t = c2.transformation
      selection_bb = entity.bounds
      
      # Place copy on Drawing layer
      c2.layer=(@layers[@dwg_layer_name])
      
      #Get Scene Name for drawing scene
      if @ans[1] == "Yes" then pn = "D - " + @dwg_layer_name[1,2] end
      if @ans[1] == "No" then pn = @user_name end
    
      # Test Scene name to determine if unique
      loop = "yes"
      while loop == "yes"
        loop = "no"
        @pages.each {|page|
          if page.name == pn
          UI.messagebox("Page \"#{pn}\" exists,make unique.")
          ps = "Make Page Name Unique!!"
          dv = pn
          ts = "Page Name for Component"
          pn = self.get_user_name(ps,dv,ts)
          loop = "yes"
          end # if page
        }
      end # while
      
      # Add Page & Description for Drawing Scene
      self.add_page(pn,@user_name,@dwg_layer_name)
      page = @pages[@user_name]
      
      #Add Component Name as Text to Drawing Scene
      name_text = @ents.add_text @user_name, [selection_bb.center.x,selection_bb.center.y,(selection_bb.center.z - (selection_bb.depth * 0.7))]
      name_text.layer=(@layers[@dwg_layer_name]) 
     
      
      #Add Title to Page Description
      #description = page.description =  scene_title
    
      #This section adds Drawing Views & Set up Drawing Scene
      @ss.clear #Clear selection so only one component will be selected
      @ss.add c2  #Add instance in Scene to selection
      self.drawing_views(ent_def,@dwg_layer_name,c2_t,c2)
      Sketchup.active_model.active_view.camera.perspective = false
      Sketchup.send_action "viewFront:"
      @view.zoom @ss
      @view.refresh
      Sketchup.send_action "pageUpdate:"
        
  Sketchup.active_model.commit_operation # Comp2_Scene
  
      @ss.clear
  
    end # comp2_Scene_command
    #----------------------------------------------------------------------------------#
    
  
    ######### Use to create New Drawing Views for an existing Drawing Scene #########
    ##### For example a Component not aligned with Global Axis that the User has fixed. #########
 
    def add_dwg_views_command
    
      self.get_model_info
      entity = @ss.first
      ent_def = entity.definition
      self.read_data
      self.main_view_check
      cName=@ss[0].definition.name
      parent = entity.parent
      cur_layer_name = @ss[0].layer.name
      self.read_data
      @user_name = @ss[0].definition.name
      @layer_name = entity.layer.name
      c2 = entity
      c2_t = c2.transformation
      selection_bb = entity.bounds
      
  Sketchup.active_model.start_operation "DwgViews"
      
      #Add Component Name as Text to Drawing Scene
      name_text = @ents.add_text @user_name, [selection_bb.center.x,selection_bb.center.y,(selection_bb.center.z - (selection_bb.depth * 0.7))]
      name_text.layer=(@layers[cur_layer_name])
      
      #Create new drawing views
      self.drawing_views(ent_def,@layer_name,c2_t,c2)
      Sketchup.active_model.active_view.camera.perspective = false
      Sketchup.send_action "viewFront:"
      @view.zoom @ss
      Sketchup.send_action "pageUpdate:"
      
  Sketchup.active_model.commit_operation# DwgViews
  
      @ss.clear
      
    end # def add_dwg_views_command
    #----------------------------------------------------------------------------------#
    
    
    ############ Used to Review &/Or Change User Preferences ##########################  
      
    def get_pref_command
    
      self.get_model_info
      # Test for Existance of Data File
      @file =  __FILE__.gsub(/.rb$/,'.txt')
      if !(FileTest.exist?(@file))
        @ans = Array.new(4)
        f = File.new(@file, "w+") 
        #Debug UI.messagebox"created file"
        # Default Data
        @ans[0] = "None"
        @ans[1] = "No"
        @ans[2] = "No"
        @ans[3] = "Assembly"
        @ans[4] = 200.inch.to_s
        f.puts @ans
        f.close
      end # if not file
      # Get Existing Preferences
      self.read_data
      prompts = ["Prefix for Comp Layer Name","Use Short Scene Name","Change Component Name","Main View Scene Name","Offset for Dwg Scene"]
      defaults= [@ans[0].strip,@ans[1].strip,@ans[2].strip,@ans[3].strip,@ans[4].strip]
      choices = ["","Yes|No","Yes|No","",""] 
      results = inputbox prompts, defaults, choices, "Input Preferences "
      if !results then;Sketchup.send_action "selectSelectionTool:";return;end
      # New User Preferences
      @ans[0] = results[0]
      @ans[1] = results[1]
      @ans[2] = results[2]
      @ans[3] = results[3]
      @ans[4] = results[4]
      begin
        @offset = results[4].to_l
        #Debug UI.messagebox("Offset " + @offset.to_s)
      rescue ### fails if @offset is empty!
          @offset = 200.inch
          @ans[4] = @offset.to_s
          UI.messagebox("WARNING: Offset invalid Number!\n\n[" + results[4] + "]   \n\nOffset was RESET to DEFAULT ")
      end
      # Save New User Preferences
      self.write_data
      self.main_view_check
         
    end # get_pref_command
    #----------------------------------------------------------------------------------#
  

############ Use HIDLAYER to add a layer that is hidden in all Scenes/Pages ############


  def hidden_Layer_command
  
    self.get_model_info
    ps = "Enter New Layer Name "
    dv = ""
    ts = "Get Name for New Layer"
    tLN = get_user_name(ps,dv,ts)
    @user_name = self.test_layerName(tLN)
    self.create_layer(@user_name)
    
  end # hidden_Layer_command

    #-----------------------------------------------------------------------------------#
    
  ######## Common def section ####################################################
  #  
    def get_model_info
    
      @model = Sketchup.active_model
      @layers = @model.layers
      @ss = @model.selection
      @ents = @model.active_entities
      @entities = @model.entities
      @pages = @model.pages
      @count = @pages.count
      @length = @layers.length
      @definitions = @model.definitions
      @view = Sketchup.active_model.active_view
    end # get_model_info
    
    def test_layerName(tLN)
      
      # Test name to determine if unique
      while @layers[tLN]
        UI.messagebox("Layer \"#{tLN}\" exists,make unique.")
        ps = "Make Layer Name Unique!!"
        dv = tLN
        ts = "Layer Name"
        tLN = self.get_user_name(ps,dv,ts)
      end # while @layers
      return tLN
      
    end # test_layerName
    
    def get_Drawing_layer_No
      
      num_layers = @model.layers.find_all { |layer| layer.name =~ /\AZ\d\d\-/ }     
      num_layer_names = num_layers.map { |layer| layer.name }
      num_layer_names.sort!
      length_n = num_layer_names.length
      #Debug UI.messagebox("num_layer_names = "  + num_layer_names.to_s)
      if length_n >0
        lname = num_layer_names[length_n-1]
        #Debug UI.messagebox("lname = "  + lname)
        name_no = lname[1,2].to_i
        next_name_no = (name_no + 1).to_s
        #Debug UI.messagebox("next_name_no = "  + next_name_no)
        if next_name_no.length < 2 
            layerno = "0" + next_name_no
        else
            layerno = next_name_no
        end # if next_name_no
      else
        layerno = "01" #Provides Starting No for New Dwg
      end # if length
      #Debug UI.messagebox("layerno = "  + layerno)
      return layerno
    
    end # get_Drawing_layer_No
    
    def get_user_name(ps,dv,ts)
      
      prompts = [ps]
      defaults= [dv]                
      results = inputbox prompts, defaults, ts
      if !results then;Sketchup.send_action "selectSelectionTool:";return;end
      user_name = results[0]
      return user_name
      
    end # get_user_name
    
    def get_bounds_max_dir(selection_bb)
    
      @max_dir = "blue axis"  #Depth
      if selection_bb.height > selection_bb.depth
        @max_dir = "green axis"
        if selection_bb.width > selection_bb.height then @max_dir = "red axis"  end
       else
        if selection_bb.width > selection_bb.depth then @max_dir = "red axis" end
      end
      
    end # get_bounds_max_dir
    
    def get_trans
    
      case @max_dir

        when "blue axis"
          @f_tran = Geom::Transformation.rotation [0,0,0], [0,1,0], 90.degrees  
          
        when "green axis"
          @f_tran = Geom::Transformation.rotation [0,0,0], [0,0,1], 90.degrees			
          
      end # case
    
    end # get_trans
    
      
    def create_layer(ln)  
      #Debug UI.messagebox"Layer Name  = " + @layer_name
      @layer = @layers.add ln
      pb = @layer.page_behavior = LAYER_IS_HIDDEN_ON_NEW_PAGES
      @model.pages.each { |p| p.set_visibility( @layer, false ) }
    
    end # create_layer
    
    def add_page(pn,pd,ln)
    
      @newpage = pn
      @pages.add @newpage
      page = @pages [@newpage]
      page.description = pd
      page.set_visibility(@layers[ln],true)
      
    end # add_page
    
    def get_page_indexNo(f_pn)
    
      page_No = 0
      page_index = -1
      @pages.each {|page|page_index = page_index + 1;
        if page.name == f_pn then  page_No = page_index end 
         }
      #Debug  UI.messagebox("Main View Page Index = " +  page_No.to_s)
      return page_No
    
    end # get_page_indexNo
    
    def drawing_views(ent_def,l_n,pt,c2)
      
      bb = c2.bounds
      w = bb.width; #UI.messagebox("Width = " + w.to_s)
      h = bb.height ; #UI.messagebox("Height = " + h.to_s)
      d = bb.depth ; #UI.messagebox("Depth = " + d.to_s)
      vvec = Z_AXIS.clone
      hvec = X_AXIS.clone

      x_d = ( h + w ) / 2.0
      z_d = ( h + d ) / 2.0

      max = [x_d*0.25, z_d*0.25].max
      x_d  += max
      z_d  += max
      
      cR=@model.active_entities.add_instance(ent_def,pt)
      cR.layer=(@layers[l_n]) #Place on new layer
      vec = hvec
      vec.length = x_d
      cR.transform! vec
      rx = Geom::Transformation.rotation(cR.bounds.center, vvec, -90.degrees)
      cR.transform! rx
      @ss.add cR  #Add instance in Scene to selection
      
      cL=@model.active_entities.add_instance(ent_def,pt)
      cL.layer=(@layers[l_n]) #Place on new layer
      vec = Geom::Vector3d.new(x_d, 0, 0).reverse
      cL.transform! vec
      rx = Geom::Transformation.rotation(cL.bounds.center, vvec, 90.degrees)
      cL.transform! rx
      @ss.add cL  #Add instance in Scene to selection
      
      cT=@model.active_entities.add_instance(ent_def,pt)
      cT.layer=(@layers[l_n]) #Place on new layer
      vec = vvec
      vec.length = z_d
      cT.transform! vec
      rx = Geom::Transformation.rotation(cT.bounds.center, hvec, 90.degrees)
      cT.transform! rx
      @ss.add cT  #Add instance in Scene to selection
      
      cBm=@model.active_entities.add_instance(ent_def,pt)
      cBm.layer=(@layers[l_n]) #Place on new layer
      vec = Geom::Vector3d.new(0, 0, z_d).reverse
      cBm.transform! vec
      rx = Geom::Transformation.rotation(cBm.bounds.center, hvec, -90.degrees)
      cBm.transform! rx
      @ss.add cBm  #Add instance in Scene to selection
      
    end # drawing_views
    
    def main_view_check
    
      # Add page for Main View if not already exists
      @mv_name = @ans[3].strip
      pagename=[]
      pgAssy = "no"
      @pages.each {|p|
        pagename = p.name
        if pagename == @mv_name
          pgAssy = "yes"
        end #if 
      } # end each loop
      
      if pgAssy == "no"
        @pages.add @mv_name
      end # if 
    
    end # main_view_check
    
    def read_data
    
      #Test for Data File
      @file =  __FILE__.gsub(/.rb$/,'.txt')
      if !(FileTest.exist?(@file))
        get_pref_command
      else
        # Read an array of Input Data from the file 
        # open input data file in read mode	       
        f = File.open(@file, "r")    
        @ans = f.readlines
        # Close the file
        f.close
        for i in 0..4
          @ans[i] = @ans[i].strip
        end # for loop
        begin
        @offset = @ans[4].to_l
        #Debug UI.messagebox("Offset " + @offset.to_s)
        rescue ### fails if @offset is empty!
          @offset = 200.inch
          @ans[4] = @offset.to_s
          UI.messagebox("WARNING: Offset invalid Number!\n\n[" + results[4] + "]   \n\nOffset was RESET to DEFAULT ")
        end
      end # if !(FileTest
    end # read_data
	
    def write_data
    
      # Write an array of Input Data to the file
      # open input data file in write mode	       
      f = File.open(@file, "w")
      f.puts @ans
      # Close the file
      f.close
      
    end # write_data
   
  end # proxy class

  ### RUN ONCE
  #
  unless file_loaded?(File.basename(__FILE__))
  
    ### CONTEXT MENU HANDLER
    #
    UI.add_context_menu_handler do |popup|
      sel = Sketchup.active_model.selection
      unless sel.empty?
        if sel.single_object?
          obj = sel[0]
          if obj.is_a?(Sketchup::ComponentInstance) &&
          obj.parent.is_a?(Sketchup::ComponentDefinition)
            popup.add_item(NC2SCENE) {
              #
              nested_component_command(obj)
              #
            }
          else
            if obj.is_a?(Sketchup::ComponentInstance)
              popup.add_item(C2LAYSCENE) {
                #
                comp2_layer_scene_command
                #
              }
              popup.add_item(C2SCNE) {
                #
                comp2_Scene_command
                #
              }
              popup.add_item(DWGVIEWS) {
                #
                add_dwg_views_command
                #
              }
            end # Single component test
          end # Nested component test
        end # single_object?
        if sel.length >= 2
          popup.add_item(GP2SCENE) {
              #
              sub_assembly_command
              #
            }
        end # sel.length
      end # unless empty?
      
    end # add_context_menu_handler
    
    UI.menu("Plugins").add_item(SETPREF){get_pref_command}
    UI.menu("Plugins").add_item(HIDLAYER){hidden_Layer_command}
    
    file_loaded(File.basename(__FILE__))
    
  end # RUN ONCE
  
end # module
